__title__ = 'songci'
__description__ = 'songci checks proxies'
__url__ = 'https://github.com/terbeznik/songci'
__version__ = '0.1.2'
__author__ = 'Alexander Terbeznik'
__author_email__ = 'alexander.terbeznik@gmail.com'
